<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH . 'libraries/REST_Controller.php';

class Api extends REST_Controller
{

    function __construct()
    {
        // Construct the parent class
        parent::__construct();
        $this->load->helper('email_send_helper');

    }


    public function signup_post(){

        $this->form_validation->set_rules('email','Email','required|valid_email|is_unique[users.email]');
        $this->form_validation->set_rules('password','Password','required');
        $this->form_validation->set_rules('name','Name','required');



        if($this->form_validation->run() == true){

            $insert_data = array(
                'name' => $this->post('name'),
                'email' => $this->post('email'),
                'password'=>  password_hash($this->post('password'), PASSWORD_DEFAULT),
                'role' => 'USER',
                'status' => 1,
            );

            $user_creation = $this->user_model->createUser($insert_data);

            if($user_creation['status']){
                $this->response([
                    'status' =>  $user_creation['status'],
                    'response' => $user_creation['message'],
                    'new_user_id' =>$user_creation['id']
                ], REST_Controller::HTTP_OK);
            }else{
                $this->response([
                    'status' =>  $user_creation['status'],
                    'response' => $user_creation['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        }else{
            $this->response([
                'status' =>  false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }


    public function login_post(){

        $this->form_validation->set_rules('email','Email','required');
        $this->form_validation->set_rules('password','Password','required');


        if($this->form_validation->run() == true){

            $login_data = array(
                'email' => $this->post('email'),
                'password' => $this->post('password'),
                'registration_id' => $this->post('registration_id')

            );


            $user_login_result = $this->user_model->validateUserLogin($login_data);
            if($user_login_result['status']){
                $this->response([
                    'status' =>  $user_login_result['status'],
                    'response' =>$user_login_result['user_login']
                ], REST_Controller::HTTP_OK);
            }else{
                $this->response([
                    'status' =>  $user_login_result['status'],
                    'response' => $user_login_result['message']
                ], REST_Controller::HTTP_NOT_FOUND);
            }

        }else{
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }


    public function facebook_login_post(){

        $this->form_validation->set_rules('facebook_id','FacebookID','required');


        if($this->form_validation->run() == true){


            $findFbUser = $this->user_model->findUserBy('facebook_id', $this->input->post('facebook_id'));
            ;
            if (count($findFbUser) > 0) {
                if ($findFbUser->status == 1) {
                    $fbExistingUser = array();

                    $fbExistingUser['user_id'] = $findFbUser->id;
                    $fbExistingUser['user_image'] = ($findFbUser->user_image != '' && file_exists(base_url() . "./uploads/200x200_" . $findFbUser->user_image)) ? base_url() . '/uploads/' . $findFbUser->user_image : '';

                    $this->response([
                        'status' => true,
                        'message' => 'Successfully Facebook Logged In',
                        'response' => $fbExistingUser
                    ], REST_Controller::HTTP_OK);
                } else {
                    $this->response([
                        'status' => false,
                        'response' => 'Your Account is disabled.Contact Admin'
                    ], REST_Controller::HTTP_BAD_REQUEST);
                }

            }else{
                $this->response([
                    'status' => false,
                    'response' => 'Invalid Facebook User'
                ], REST_Controller::HTTP_BAD_REQUEST);
            }

        }else{
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }


    public function facebook_register_post(){

        $this->form_validation->set_rules('name','Name','required');
        $this->form_validation->set_rules('email','Email','required|is_unique[users.email]');
        $this->form_validation->set_rules('facebook_id','Facebook ID','required');
        $this->form_validation->set_rules('is_silhouette','Is Silhouette','required');


        if($this->form_validation->run() == true) {
            $findFbUser = $this->user_model->findUserBy('facebook_id', $this->input->post('facebook_id'));
            if (count($findFbUser) > 0) {
                if ($findFbUser->status == 1) {
                    $fbExistingUser = array();

                    $fbExistingUser['user_id'] = $findFbUser->id;
                    $fbExistingUser['user_image'] = ($findFbUser->user_image != '' && file_exists(base_url() . "./uploads/200x200_" . $findFbUser->user_image)) ? base_url() . '/uploads/' . $findFbUser->user_image : '';

                    $this->response([
                        'status' => true,
                        'response' => $fbExistingUser
                    ], REST_Controller::HTTP_OK);
                } else {
                    $this->response([
                        'status' => false,
                        'response' => 'Your Account is disabled.Contact Admin'
                    ], REST_Controller::HTTP_BAD_REQUEST);
                }

            } else {
                if ($this->input->post('is_silhouette') == -1) {
                    $large_image_url = 'https://graph.facebook.com/' . $this->input->post('facebook_id') . '/picture?type=large';
                    $idle_image_url = 'https://graph.facebook.com/' . $this->input->post('facebook_id') . '/picture?width=200&height=200';
                    $large_image_content = file_get_contents($large_image_url);
                    file_put_contents( "./uploads/" . $this->input->post('facebook_id') . '.jpg', $large_image_content);
                    $idle_image_content = file_get_contents($idle_image_url);
                    file_put_contents("./uploads/200x200_" . $this->input->post('facebook_id') . '.jpg', $idle_image_content);
                    file_put_contents("./uploads/500x500_" . $this->input->post('facebook_id') . '.jpg', $large_image_content);
                    $user_image = $this->input->post('facebook_id') . '.jpg';
                } else {
                    $user_image = '';
                }

                $facebook_register_data = array(
                    'name' => $this->post('name'),
                    'email' => $this->post('email'),
                    'facebook_id' => $this->post('facebook_id'),
                    'user_image' => $user_image,
                    'status' => 1

                );

                $user_creation = $this->user_model->createUser($facebook_register_data);

                if ($user_creation['status']) {
                    $this->response([
                        'status' => $user_creation['status'],
                        'response' => $user_creation['message'],
                        'new_user_id' => $user_creation['id']
                    ], REST_Controller::HTTP_OK);
                } else {
                    $this->response([
                        'status' => $user_creation['status'],
                        'response' => $user_creation['message']
                    ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
                }
            }
        }else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function google_login_post(){

        $this->form_validation->set_rules('google_id','GoogleID','required');


        if($this->form_validation->run() == true){


            $findGoogleUser = $this->user_model->findUserBy('google_id', $this->input->post('google_id'));
            ;
            if (count($findGoogleUser) > 0) {
                if ($findGoogleUser->status == 1) {
                    $googleExistingUser = array();

                    $googleExistingUser['user_id'] = $findGoogleUser->id;
                    $googleExistingUser['user_image'] = ($findGoogleUser->user_image != '' && file_exists(base_url() . "./uploads/200x200_" . $findGoogleUser->user_image)) ? base_url() . '/uploads/' . $findGoogleUser->user_image : '';

                    $this->response([
                        'status' => true,
                        'message' => 'Successfully Google Logged In',
                        'response' => $googleExistingUser
                    ], REST_Controller::HTTP_OK);
                } else {
                    $this->response([
                        'status' => false,
                        'response' => 'Your Account is disabled.Contact Admin'
                    ], REST_Controller::HTTP_BAD_REQUEST);
                }

            }else{
                $this->response([
                    'status' => false,
                    'response' => 'Invalid Google User'
                ], REST_Controller::HTTP_BAD_REQUEST);
            }

        }else{
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function google_register_post(){

        $this->form_validation->set_rules('name','Name','required');
        $this->form_validation->set_rules('email','Email','required|is_unique[users.email]');
        $this->form_validation->set_rules('google_id','Google ID','required');


        if($this->form_validation->run() == true) {
            $findGoogleUser = $this->user_model->findUserBy('google_id', $this->input->post('google_id'));
            if (count($findGoogleUser) > 0) {
                if ($findGoogleUser->status == 1) {
                    $googleExistingUser = array();

                    $googleExistingUser['user_id'] = $findGoogleUser->id;
                    $googleExistingUser['user_image'] = ($findGoogleUser->user_image != '' && file_exists(base_url() . "./uploads/200x200_" . $findGoogleUser->user_image)) ? base_url() . '/uploads/' . $findGoogleUser->user_image : '';

                    $this->response([
                        'status' => true,
                        'response' => $googleExistingUser
                    ], REST_Controller::HTTP_OK);
                } else {
                    $this->response([
                        'status' => false,
                        'response' => 'Your Account is disabled.Contact Admin'
                    ], REST_Controller::HTTP_BAD_REQUEST);
                }

            } else {
                if ($this->input->post('user_image')) {
                    file_put_contents(base_url() . "uploads/" . $this->input->post('google_id') . '.jpg', $this->input->post('user_image'));
                    file_put_contents(base_url() . "uploads/200x200_" . $this->input->post('google_id') . '.jpg', $this->input->post('user_image'));
                    file_put_contents(base_url() . "uploads/500x500_" . $this->input->post('google_id') . '.jpg', $this->input->post('user_image'));
                    $user_image = $this->input->post('google_id') . '.jpg';
                } else {
                    $user_image = '';
                }

                $google_register_data = array(
                    'name' => $this->post('name'),
                    'email' => $this->post('email'),
                    'google_id' => $this->post('google_id'),
                    'user_image' => $user_image,
                    'status' => 1

                );

                $user_creation = $this->user_model->createUser($google_register_data);

                if ($user_creation['status']) {
                    $this->response([
                        'status' => $user_creation['status'],
                        'response' => $user_creation['message'],
                        'new_user_id' => $user_creation['id']
                    ], REST_Controller::HTTP_OK);
                } else {
                    $this->response([
                        'status' => $user_creation['status'],
                        'response' => $user_creation['message']
                    ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
                }
            }
        }else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function get_products_post(){

        $id = $this->get('id');
        $this->form_validation->set_rules('userId','UserID','required');
      

        if($this->form_validation->run() == true){

            $stock_status = $this->input->post('stock_status');

            if($stock_status != ''){
                if(!$this->stockStatusArray($stock_status)){
                     $this->response([
                'status' => false,
                'response' =>  'Not a valid stock status. valid values are out_of_stock/in_stock'
            ], REST_Controller::HTTP_BAD_REQUEST);
                }
            }

            if($id == ''){
                $categories = $this->products_model->getProducts('','','',$stock_status);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' =>$categories['products']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }

            }else{
                $categories = $this->products_model->getProducts($id,'','',$stock_status);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' =>$categories['products']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'message' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }
            }
        }else{
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function getproductupdatescount_post(){

        $this->form_validation->set_rules('userId','UserID','required');
        
        if ($this->form_validation->run() == true) {
             $user_id = $this->post('userId');

            
                $categories = $this->products_model->getProductUpdates($user_id);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' => array(
                            'updates_count' => count($categories['updates']))
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }   
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function setupdateread_post(){
        $this->form_validation->set_rules('userId','UserID','required');
      
    


        if ($this->form_validation->run() == true) {

             $data = array(
                'user_id' => $this->post('userId'),
                
            );

            $question_notes_update = $this->products_model->setupdateread($data);

            if ($question_notes_update) {
                $this->response([
                    'status' => true,
                    'response' => "Successfully updated as read"
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => false,
                    'response' => "Failed to update as read"
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }


    function stockStatusArray($str)
    {
        $possible_values = array('out_of_stock', 'in_stock');
        if(in_array($str, $possible_values)){
            return true;
        }
                 return false;
    }

        public function registerfcm_post(){
            $data = array();
            $this->load->library('form_validation');
            $this->form_validation->set_rules('device_id', 'Device ID', 'trim|required');
            $this->form_validation->set_rules('token', 'Token', 'trim|required');
            $this->form_validation->set_rules('device', 'Device', 'trim|required');
            if ($this->form_validation->run() == FALSE) 
        {
                 $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }else
            {   
                $device = $this->input->post("device");
                $token = $this->input->post("token");
                $device_id = $this->input->post("device_id");
                
                $field = "";
                if($device=="android"){
                    $field = "user_gcm_code";
                }else if($device=="ios"){
                    $field = "user_ios_token";
                }
                $q = $this->db->query("select device_id from users where device_id = '".$device_id."'");
                if($q->num_rows() > 0){
                    if($field!=""){
                    $this->db->query("update users set ".$field." = '".$token."' where device_id = '".$device_id."'");
                    $this->response([
                        'status' => true,
                        'response' =>  'Successfully updated fcm'], REST_Controller::HTTP_OK);
                    $data["responce"] = true;    
                }else{
                    $this->response([
                        'status' => true,
                        'response' =>  'Device type is not set'], REST_Controller::HTTP_BAD_REQUEST);
                }
            }else{
                if($field!=""){
                    $insert_data = array(
                        $field => $token,
                        'device_id' => $device_id
                    );
                    $this->db->insert('users',$insert_data);
                    $this->response([
                        'status' => true,
                        'response' =>  'Successfully updated fcm for new device'], REST_Controller::HTTP_OK);
                    $data["responce"] = true;    
                }else{
                    $this->response([
                        'status' => true,
                        'response' =>  'Device type is not set'], REST_Controller::HTTP_BAD_REQUEST);
                }
            }
                
                
                
            }
    }

    /** below not needed as of now */

    public function get_questions_post(){

        $this->form_validation->set_rules('userId','UserID','required');
        if($this->input->post('question_state') != ''){
            $this->form_validation->set_rules('question_state','Question State','callback_stateInArray');
        }
       
        if($this->form_validation->run() == true){
            $category_id = $this->input->post('category_id');
            $limit = ($this->input->post('total_required_question_count') != '')
                ? $this->input->post('total_required_question_count') : '';
            $question_state = ($this->input->post('question_state') != '')
                ? $this->input->post('question_state') : '';

            if($category_id != ''){
                $categories = $this->question_model->getQuestions($category_id,$limit,'',$question_state);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' =>$categories['questions']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }

            }else{
                $categories = $this->question_model->getQuestions('',$limit,'',$question_state);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' =>$categories['questions']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'message' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }
            }
        }else{
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function get_question_notes_post(){

        $this->form_validation->set_rules('userId','UserID','required');
        $this->form_validation->set_rules('question_id','Question ID','required');


        if($this->form_validation->run() == true){


                $questionNotes = $this->question_model->getUserNotes('',$this->post('question_id'));
                if($questionNotes['status']){
                    $this->response([
                        'status' =>  $questionNotes['status'],
                        'response' =>$questionNotes['user_notes']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $questionNotes['status'],
                        'message' => 'No notes found'
                    ], REST_Controller::HTTP_NOT_FOUND);
                }
        }else{
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    } 


    public function get_questionwise_notes_post(){

        $this->form_validation->set_rules('userId','UserID','required');
        $this->form_validation->set_rules('question_id','Question ID','required');


        if($this->form_validation->run() == true){


                $questionNotes = $this->question_model->getQuestionNotes($this->post('question_id'));
                if($questionNotes['status']){
                    $this->response([
                        'status' =>  $questionNotes['status'],
                        'response' =>$questionNotes['user_notes']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $questionNotes['status'],
                        'message' => 'No notes found'
                    ], REST_Controller::HTTP_NOT_FOUND);
                }
        }else{
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function get_counts_post(){

        $this->form_validation->set_rules('userId','UserID','required');


        if($this->form_validation->run() == true){

            $cat_id = ($this->post('category_id') != '') ? $this->post('category_id') : '';


                $questionNotes = $this->question_model->getCounts($cat_id);
                if($questionNotes['status']){
                    $this->response([
                        'status' =>  $questionNotes['status'],
                        'response' =>$questionNotes['counts']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $questionNotes['status'],
                        'message' => 'No counts'
                    ], REST_Controller::HTTP_NOT_FOUND);
                }
        }else{
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function add_notes_post(){
        $this->form_validation->set_rules('userId','UserID','required');
        $this->form_validation->set_rules('question_id','Question ID','required');
        $this->form_validation->set_rules('note','Note','required');
        $this->form_validation->set_rules('note_type','Note Type - user_note/feedback/report','required');


        if ($this->form_validation->run() == true) {

            $question_notes_data = array(
                'question_id' => $this->post('question_id'),
                'user_id' => $this->post('userId'),
                'note' => $this->post('note'),
                'note_type' => $this->post('note_type'),
            );

            $question_notes_update = $this->question_model->postQuestionNotes($question_notes_data);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['response']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }

    }

    public function add_questionnotes_post(){
        $this->form_validation->set_rules('userId','UserID','required');
        $this->form_validation->set_rules('question_id','Question ID','required');
        $this->form_validation->set_rules('note','Note','required');
        $this->form_validation->set_rules('privacy','privacy','required|callback_validquestionnotesprivacy');


        if ($this->form_validation->run() == true) {

            $question_notes_data = array(
                'question_id' => $this->post('question_id'),
                'user_id' => $this->post('userId'),
                'note' => $this->post('note'),
                'privacy' => $this->post('privacy'),
            );


            $question_notes_update = $this->question_model->postQuestionWiseNotes($question_notes_data);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['response']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }

    }

    public function set_user_result_post(){
        $this->form_validation->set_rules('userId','UserID','required');
        $this->form_validation->set_rules('result_data','Result Data','required|callback_validjsonchecker');


        if ($this->form_validation->run() == true) {

            $json_decoded =  json_decode($this->post('result_data'),true);

            $question_notes_update = $this->user_model->postUserResult($this->post('userId'),json_encode(json_decode($this->post('result_data'))),$json_decoded);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }

    }


    public function get_statistics_post(){
        $this->form_validation->set_rules('userId','UserID','required');

        if ($this->form_validation->run() == true) {


            $getUserStatistics = $this->user_model->getUserStatistics($this->input->post('userId'));

            if ($getUserStatistics['status']) {
                $this->response([
                    'status' => $getUserStatistics['status'],
                    'response' => $getUserStatistics['response']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $getUserStatistics['status'],
                    'response' => $getUserStatistics['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }

    }

    public function forgot_password_post(){
        $this->form_validation->set_rules('email','Email','required');


        if ($this->form_validation->run() == true) {
            $findUser = $this->user_model->findUserBy('email', $this->input->post('email'));

            if(count($findUser) > 0){
                if($findUser->status == 1){
                    $password_reset_token = uniqid();

                    $update_data = array(
                        'password_reset_token' => $password_reset_token
                    );

                    $updateUserData = $this->user_model->updateUserData($update_data,$findUser->id);

                    if ($updateUserData['status']) {

                        $email_data['to_address']      = array($this->input->post('email')); // Send email to our user
                        $email_data['subject'] = 'FlashCard Password Reset'; // Give the email a subject
                        $email_data['html'] = '
You requested for reset password.
------------------------
Password reset code : '.$password_reset_token.'

------------------------ 
';
                        emailSender($email_data);

                        $this->response([
                            'status' => $updateUserData['status'],
                            'response' => $updateUserData['message']
                        ], REST_Controller::HTTP_OK);
                    } else {
                        $this->response([
                            'status' => $updateUserData['status'],
                            'response' => $updateUserData['message']
                        ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
                    }
                }else{
                    $this->response([
                        'status' => false,
                        'response' => 'Your Account is disabled.Contact Admin'
                    ], REST_Controller::HTTP_BAD_REQUEST);
                }
            }else{
                $this->response([
                    'status' => false,
                    'response' => 'User not found with corresponding email'
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }

    }


    public function reset_password_post(){
        $this->form_validation->set_rules('verification_code','Password Verification Code','required');
        $this->form_validation->set_rules('password','Password','required');


        if ($this->form_validation->run() == true) {
            $findUser = $this->user_model->findUserBy('password_reset_token', $this->input->post('verification_code'));

            if(count($findUser) > 0){
                if($findUser->status == 1){

                    $update_data = array(
                        'password' => password_hash($this->post('password'), PASSWORD_DEFAULT),
                        'password_reset_token' => '',
                    );

                    $updateUserData = $this->user_model->updateUserData($update_data,$findUser->id);

                    if ($updateUserData['status']) {

                        $this->response([
                            'status' => $updateUserData['status'],
                            'response' => $updateUserData['message']
                        ], REST_Controller::HTTP_OK);
                    } else {
                        $this->response([
                            'status' => $updateUserData['status'],
                            'response' => $updateUserData['message']
                        ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
                    }
                }else{
                    $this->response([
                        'status' => false,
                        'response' => 'Your Account is disabled.Contact Admin'
                    ], REST_Controller::HTTP_BAD_REQUEST);
                }
            }else{
                $this->response([
                    'status' => false,
                    'response' => 'User not found with corresponding email'
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }

    }

    public function updateprofile_post(){

        $this->form_validation->set_rules('firstname','Firstname','required');
        $this->form_validation->set_rules('lastname','Lastname','required');
        $this->form_validation->set_rules('address','Address','required');
        $this->form_validation->set_rules('city','City','required');
        $this->form_validation->set_rules('state','State','required');
        $this->form_validation->set_rules('country','Country','required');
        $this->form_validation->set_rules('phone_number','Phone Number','required');
        $this->form_validation->set_rules('description','Description','required');

        $id = $this->get('id');

        if($id == ''){
            $this->response([
                'status' => FALSE,
                'message' => 'UserId cannot be empty.'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }else {
            if ($this->form_validation->run() == true) {
                $config['upload_path'] = './uploads/';
                $config['allowed_types'] = 'jpg';
                $config['max_size'] = '2048000';
                $config['encrypt_name'] = TRUE;

                $this->load->library('upload', $config);

                if (!$this->upload->do_upload('profile_picture'))
                {
                    $this->response([
                        'status' => false,
                        'response' => $this->upload->display_errors('', '')
                    ], REST_Controller::HTTP_BAD_REQUEST);

                }else{
                    $upload_data = $this->upload->data();

                    $update_data = array(
                        'firstname' => $this->post('firstname'),
                        'lastname' => $this->post('lastname'),
                        'address' => $this->post('address'),
                        'city' => $this->post('city'),
                        'state' => $this->post('state'),
                        'country' => $this->post('country'),
                        'phone_number' => $this->post('phone_number'),
                        'description' => $this->post('description'),
                        'profile_picture' =>  $upload_data['file_name']

                    );

                    $user_profile_updation = $this->user_model->updateUserProfile($update_data, $id);

                    if ($user_profile_updation['status']) {
                        $this->response([
                            'status' => $user_profile_updation['status'],
                            'response' => $user_profile_updation['message']
                        ], REST_Controller::HTTP_OK);
                    } else {
                        $this->response([
                            'status' => $user_profile_updation['status'],
                            'response' => $user_profile_updation['message']
                        ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
                    }
                }

            } else {
                $this->response([
                    'status' => false,
                    'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
                ], REST_Controller::HTTP_BAD_REQUEST);
            }
        }
    }


    public function setquestionstate_post(){
        $this->form_validation->set_rules('userId','UserID','required');
        $this->form_validation->set_rules('question_id','Question ID','required');
        $this->form_validation->set_rules('question_state','Question State','required');


        if ($this->form_validation->run() == true) {

            $question_notes_data = array(
                'question_id' => $this->post('question_id'),
                'user_id' => $this->post('userId'),
                'question_state' => $this->post('question_state'),
            );

            $question_notes_update = $this->question_model->setQuestionState($question_notes_data);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['response']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }
    public function getquestionstates_post(){
        $this->form_validation->set_rules('userId','UserID','required');
        $this->form_validation->set_rules('question_state','Question State','required');


        if ($this->form_validation->run() == true) {

            $where = array(
                'user_id' => $this->post('userId'),
                'question_state' => $this->post('question_state'),
                'category_id' => $this->post('category_id'),
            );

            $question_notes_update = $this->question_model->getQuestionStates($where);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'no_of_questions' => count($question_notes_update['questions']),
                    'response' =>  $question_notes_update['questions']
                    
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function getquestionstatescount_post(){
        $this->form_validation->set_rules('userId','UserID','required');
        $this->form_validation->set_rules('question_state','Question State','required');


        if ($this->form_validation->run() == true) {

            $where = array(
                'user_id' => $this->post('userId'),
                'question_state' => $this->post('question_state'),
                'category_id' => $this->post('category_id'),
            );

            $question_notes_update = $this->question_model->getQuestionStatesCount($where);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['questions']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }
 
 public function clearuserstats_post(){
        $this->form_validation->set_rules('userId','UserID','required');
    


        if ($this->form_validation->run() == true) {


            $user_id = $this->post('userId');

            $question_notes_update = $this->user_model->clearUserStats($user_id);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function getupdateavailable_post(){
        $this->form_validation->set_rules('userId','UserID','required');
    


        if ($this->form_validation->run() == true) {


            $user_id = $this->post('userId');

            $question_notes_update = $this->user_model->getUpdateAvailable($user_id);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['data']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function getupdateavailablelist_post(){
        $this->form_validation->set_rules('userId','UserID','required');
    


        if ($this->form_validation->run() == true) {


            $user_id = $this->post('userId');

            $question_notes_update = $this->question_model->getQuestions('','','','',true);

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['questions']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

    public function setupdateddone_post(){
        $this->form_validation->set_rules('userId','UserID','required');
    


        if ($this->form_validation->run() == true) {


            $user_id = $this->post('userId');

            $question_notes_update = $this->user_model->setupdateddone();

            if ($question_notes_update['status']) {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => $question_notes_update['status'],
                    'response' => $question_notes_update['message']
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    } 


    public function contactfeedbackemail_post(){
        $this->form_validation->set_rules('name','Name','required');
        $this->form_validation->set_rules('email','Email','valid_email|required');
        $this->form_validation->set_rules('subject','Subject','required');
        $this->form_validation->set_rules('message','Message','required');
    


        if ($this->form_validation->run() == true) {

            $email_data['from_email']      = $this->input->post('email'); // from email
            $email_data['from_name']      = $this->input->post('name'); // from name
            $email_data['to_address']      = $this->config->item('admin_receiver_email'); // Send email to admin
            $email_data['subject'] = $this->input->post('subject'); // Give the email a subject
            
            $email_data['html'] = $this->input->post('message');
            

            if (emailSender($email_data)) {
                $this->response([
                    'status' => true,
                    'response' => 'Email Successfully sent'
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => false,
                    'response' => 'Failed to send'
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }



    function validjsonchecker($json_str)
    {
        $return_value = json_decode($json_str);
        if ($return_value == null)
        {
            $this->form_validation->set_message('validjsonchecker', '%s is not valid JSON string');
            return FALSE;
        }
        else
        {
            return TRUE;
        }
    }

    public function stateInArray($str)
    {
        $possible_values = array('unanswered', 'saved', 'incorrect','correct');
        if(in_array($str, $possible_values)){return true;}
         $this->form_validation->set_message('stateInArray', '%s is not valid question state.possible values are unanswered/saved/incorrect/correct');
            return FALSE;
        return false;
    }
 public function validquestionnotesprivacy($str)
    {
        $possible_values = array('public', 'private');
        if(in_array($str, $possible_values)){return true;}
         $this->form_validation->set_message('validquestionnotesprivacy', '%s is not valid Notes privacy.possible values are public/private');
            return FALSE;
        return false;
    }

    public function getupdates_post(){
        $id = $this->get('id');

       
            $user_id = $this->post('userId');
          
            if($id == ''){
                $categories = $this->question_model->getUpdates($user_id);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' => $categories['updates']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }

            }else{
                $categories = $this->question_model->getUpdates($user_id,$id);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' =>$categories['updates']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'message' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }
            }
        
    }

    public function getupdatescount_post(){
        $id = $this->get('id');

        $this->form_validation->set_rules('userId','UserID','required');
        
        if ($this->form_validation->run() == true) {
             $user_id = $this->post('userId');

            if($id == ''){
                $categories = $this->question_model->getUpdates($user_id);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' => array(
                            'updates_count' => count($categories['updates']))
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }

            }else{
                $categories = $this->question_model->getUpdates($user_id,$id);
                if($categories['status']){
                    $this->response([
                        'status' =>  $categories['status'],
                        'response' =>$categories['updates']
                    ], REST_Controller::HTTP_OK);
                }else{
                    $this->response([
                        'status' =>  $categories['status'],
                        'message' => $categories['message']
                    ], REST_Controller::HTTP_NOT_FOUND);
                }
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

     public function getaboutpage_post(){
        $this->form_validation->set_rules('userId','UserID','required');
    


        if ($this->form_validation->run() == true) {

            $question_notes_update = $this->category_model->getabout(1);

            if ($question_notes_update) {
                $this->response([
                    'status' => true,
                    'response' => array( 'about' => $question_notes_update->about)
                ], REST_Controller::HTTP_OK);
            } else {
                $this->response([
                    'status' => false,
                    'response' => array('about' => NULL)
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
            }
        } else {
            $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
        }
    }

     /* chat */
    public function messageadmin_post(){

         $this->load->library('form_validation');
         $this->form_validation->set_rules('user_id', 'user_id', 'trim|required');
         $this->form_validation->set_rules('message', 'message', 'trim|required');

            if ($this->form_validation->run() == FALSE) 
        {

             $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
                                
        }else
            {   
                
                $user_id = $this->input->post("user_id");
                $message = $this->input->post("message");

               
                
               if($this->user_model->messageadmin($user_id,$message)){
                  $this->response([
                    'status' => true,
                    'response' => "Successfully message sent"
                ], REST_Controller::HTTP_OK);
            
               }else{
                $this->response([
                    'status' => false,
                    'response' => "Failed to send message"
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
               }
                
                
            }


    }


    public function getadminmessages_post(){

         $this->load->library('form_validation');
         $this->form_validation->set_rules('user_id', 'user_id', 'trim|required');

            if ($this->form_validation->run() == FALSE) 
        {
                $this->response([
                'status' => false,
                'response' => validation_errors() ? validation_errors() : 'Invalid Inputs'
            ], REST_Controller::HTTP_BAD_REQUEST);
                                
        }else
            {   
                
                $user_id = $this->input->post("user_id");

               
                
               if($messages = $this->user_model->getadminmessages($user_id)){

                $this->response([
                    'status' => true,
                    'response' => $messages
                ], REST_Controller::HTTP_OK);
            
               
               }else{
                 $this->response([
                    'status' => false,
                    'response' => "Failed to fetch messages"
                ], REST_Controller::HTTP_INTERNAL_SERVER_ERROR);
               }
                
                
            }
            
    }  
    /* end chat */




}